import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.text.*;
import java.util.*;

public class AVIFRate extends JFrame implements ActionListener {
    private JLabel lblFileName, lblCurFRate, lblNewFRate, lblInfo1, lblInfo2;
    private JTextField txtFileName, txtCurFRate, txtNewFRate;
    private JButton btnChooseFile, btnQuit, btnApply;
    private JPanel pnlMain, pnlButtons;
    
    private final int DEF_SCALE = 100000;
    
    public AVIFRate() {
        super("AVIFRate Java Edition 1.0.1 by Sparrow");
        
        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        
        lblFileName = new JLabel("AVI File Name:");
        txtFileName = new JTextField(27);
        txtFileName.setEditable(false);
        
        btnChooseFile = new JButton("Open AVI File...");
        btnChooseFile.setMnemonic('O');
        btnChooseFile.setIcon(new ImageIcon(getClass().getResource("res/img/folder.png")));
        btnChooseFile.addActionListener(this);
        
        lblCurFRate = new JLabel("Current Frame Rate:");
        txtCurFRate = new JTextField(10);
        txtCurFRate.setEditable(false);
        lblNewFRate = new JLabel("New Frame Rate:");
        txtNewFRate = new JTextField(10);
        
        lblInfo1 = new JLabel("(C) 09/2004 - Sparrow - http://sparrow.webhop.net/", JLabel.CENTER);
        lblInfo2 = new JLabel("Based on avifrate v1.10 by Blight", JLabel.CENTER);
        
        pnlMain = new JPanel(new GridLayout(9, 1));
        
        pnlMain.add(lblFileName);
        pnlMain.add(txtFileName);
        pnlMain.add(btnChooseFile);
        pnlMain.add(lblCurFRate);
        pnlMain.add(txtCurFRate);
        pnlMain.add(lblNewFRate);
        pnlMain.add(txtNewFRate);
        pnlMain.add(lblInfo1);
        pnlMain.add(lblInfo2);
        
        container.add(pnlMain, BorderLayout.NORTH);
        
        btnQuit = new JButton("Close");
        btnQuit.setMnemonic('C');
        btnQuit.setIcon(new ImageIcon(getClass().getResource("res/img/closewindow.png")));
        btnQuit.addActionListener(this);
        
        btnApply = new JButton("Apply");
        btnApply.setMnemonic('A');
        btnApply.setIcon(new ImageIcon(getClass().getResource("res/img/confirm.png")));
        btnApply.addActionListener(this);
        
        pnlButtons = new JPanel(new GridLayout(1, 2));
        pnlButtons.add(btnApply);
        pnlButtons.add(btnQuit);
        
        container.add(pnlButtons, BorderLayout.SOUTH);
        
        setIconImage(new ImageIcon(getClass().getResource("res/img/avifrate.png")).getImage());
        setSize(310, 280);
        setResizable(false);
        centerFrameOnScreen(this);
        setVisible(true);
    }
    
    public static void main(String args[]) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
        
        Font defFont = new DefaultFont().getFont(Font.PLAIN, 11);
        
        UIManager.put("Label.font", defFont);
        UIManager.put("Button.font", defFont);
        
        AVIFRate app = new AVIFRate();
        
        app.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
    
    private void centerFrameOnScreen(JFrame frame) {
        Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
        Dimension f=frame.getSize();
        
        frame.setBounds(Math.max(0, (d.width - f.width) / 2),
        Math.max(0, (d.height - f.height) / 2),
        f.width, f.height);
    }
    
    public void actionPerformed(ActionEvent e) {
        JFileChooser fc = new JFileChooser();
        fc.setFileFilter(new AviFileFilter());
        
        if (e.getSource() == btnQuit)
            System.exit(0);
        
        if (e.getSource() == btnChooseFile) {
            if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                String fr = null;
                txtFileName.setText(fc.getSelectedFile().getAbsolutePath());
                fr = getFrameRate(txtFileName.getText());
                txtCurFRate.setText(fr);
                txtNewFRate.setText(fr);
            }
        }
        
        if (e.getSource() == btnApply) {
            if (txtFileName.getText().length() > 0) {
                try {
                    float fr;
                    fr = Float.parseFloat(txtNewFRate.getText());
                    if (setFrameRate(txtFileName.getText(), fr)) {
                       txtCurFRate.setText(getFrameRate(txtFileName.getText()));
                       JOptionPane.showMessageDialog(this, "The new frame rate was set successfuly!", "Message", JOptionPane.INFORMATION_MESSAGE);
                    }
                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(this, "The new frame rate is not a valid number! Try again!", "Error", JOptionPane.ERROR_MESSAGE);
                }
            } else {
                JOptionPane.showMessageDialog(this, "You must select an AVI file first!", "Attention", JOptionPane.INFORMATION_MESSAGE);
            }
        }
    }
    
    private String getFrameRate(String fileName) {
        try {
            RandomAccessFile file = new RandomAccessFile(fileName, "r");
            int scale, rate;
            DecimalFormat fourDigits = new DecimalFormat("0.0000");
            fourDigits.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.US));
            
            file.seek(128);
            
            scale = swapEndian(file.readInt());
            rate  = swapEndian(file.readInt());
            
            file.close();
            
            return fourDigits.format((double) rate/scale);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, "Error trying to read AVI's Frame Rate!", "Error", JOptionPane.ERROR_MESSAGE);
            return "";
        }
    }
    
    private boolean setFrameRate(String fileName, float fr) {
        try {
            RandomAccessFile file = new RandomAccessFile(fileName, "rw");
            
            file.seek(128);
            
            file.writeInt(swapEndian(DEF_SCALE));
            file.writeInt(swapEndian(Math.round(fr * DEF_SCALE)));
            
            file.close();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, "Error trying to set AVI's Frame Rate!", "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        
        return true;
    }
    
    private int swapEndian(int v) {
        return (v >>> 24) | (v << 24) | ((v << 8) & 0x00FF0000) | ((v >> 8) & 0x0000FF00);
    }
}
